/**
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of ADC demo for the
 *          TOSHIBA 'TMPM061' Demo
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* include file */
#include <stdio.h>

#include "tmpm061_adc.h"
#include "led.h"
#include "lcd.h"
#include "retarget.h"

/* define the ADC channel used here.the VR2 is connected to AIN0 in this example */
#define ADC_CHANNEL     ADC_AN_0
#define ADC_REG_RESULT  ADC_REG_0

struct mySoftTimer {
    uint8_t flag_TimeUp;
    uint8_t flag_Blink;
    uint32_t timer_Cnt;
}softT;

#define DELAYCONST  20000U
static void softTimer_Init(void);
static void softTimer_Run(void); 
   
static void display_AD(uint32_t myResult);   

ADC_Result adResult = { 0U };
uint32_t timeUp = 0U;
uint32_t idx = 0U;
uint32_t cnt = 0U;

WorkState adcState = BUSY;
uint32_t myResult = 0U;


int main()
{
    
    /* 1. set ADC clock */
    ADC_SetClk(ADC_CONVERSION_81_CLOCK, ADC_FC_DIVIDE_LEVEL_16);

    /* 2. turn on reference voltage, and wait at least 3uS to let it stable */
    ADC_SetVref(ENABLE);
    timeUp = 300U;
    while( timeUp ) {
        timeUp -- ;
    }
    
    /* 3. use Fixed-channel single conversion */
    ADC_SetScanMode(DISABLE);
    ADC_SetRepeatMode(DISABLE);

    /* 4. select ADC channel */
    ADC_SetInputChannel(ADC_CHANNEL);

    /* 5. now start ADC */
    ADC_Start();

    /* initialize and retarget the 'printf()' to UART */
    Retarget_Init();
    
    /* initialize soft timer */
    softTimer_Init();

    while (1U) {
        softTimer_Run(); 
        if(softT.flag_TimeUp) {
            softT.flag_TimeUp = 0U;
            
            /* check ADC module state */
            adcState = ADC_GetConvertState();
            if (adcState == DONE) {
                /* read ADC result when it is finished */
                adResult = ADC_GetConvertResult(ADC_REG_RESULT);
                myResult = (uint32_t)adResult.Bit.ADResult ;
                ADC_Start();
                display_AD(myResult);
            }     
        } else {
            /* Do nothing */
        } 

    }
}


/**
  * @brief  Initialize a several mS timer. 
  * @param  None
  * @retval None
  */
static void softTimer_Init(void)
{
    softT.flag_TimeUp = 0U;
    softT.flag_Blink = 0U;
    softT.timer_Cnt = DELAYCONST;
}  
  

/**
  * @brief  Simulate a several mS timer. 
  * @param  None
  * @retval None
  */
static void softTimer_Run(void)
{
    if( softT.timer_Cnt == 0U ){
        softT.flag_TimeUp = 1U;
        softT.flag_Blink = (uint8_t)~softT.flag_Blink; 
        softT.timer_Cnt = DELAYCONST;
    } else {
        softT.timer_Cnt --;
    }
}

/**
  * @brief  display AD value on LCD. 
  * @param  myResult: AD data which will be displayed
  * @retval None
  */
void display_AD(uint32_t myResult)
{

    if( myResult > 1023U ) {
        myResult = 1023U;
    } else {
        /* Do nothing */
    }

    printf("ADC Value is %4d\r\n", myResult); 
}    

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
