/**
 *******************************************************************************
 * @file    cg.c
 * @brief   The driver functions of CG for the TOSHIBA 'TMPM061' Device Series 
 * @version V0.102
 * @date    2014/06/09
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/* Includes ------------------------------------------------------------------*/
#include "cg.h"
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define CG_WUODR_EXT     ((uint16_t)0x0FFF)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Global variables ----------------------------------------------------------*/
/* external variables --------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void switchFromIHOSCtoEHOSC(void);
void switchFromEHOSCtoIHOSC(void);
/* Private functions ---------------------------------------------------------*/

/* CG_SetSystem */
void CG_SetSystem(void)
{
    /* Switch over from IHOSC to EHOSC*/
    switchFromIHOSCtoEHOSC();

    /* Set fgear = fc/2 */
    CG_SetFgearLevel(CG_DIVIDE_2);
    /* Set fperiph to fgear */
    CG_SetPhiT0Src(CG_PHIT0_SRC_FGEAR);
    CG_SetPhiT0Level(CG_DIVIDE_64);

    /* Set low power consumption mode stop */
    CG_SetSTBYMode(CG_STBY_MODE_STOP);
    /* Set pin status in stop mode to "active" */
    CG_SetPinStateInStopMode(ENABLE);
}

/* CG_NormalToStop */
void CG_NormalToStop(void)
{
    /* Switch over from EHOSC to IHOSC*/
    switchFromEHOSCtoIHOSC();
    /* Set CG module: Normal ->Stop mode */
    CG_SetWarmUpTime(CG_WARM_UP_SRC_OSC2,CG_WUODR_EXT);
    /* Enter stop mode */
    __WFI();
}

/* Switch over from IHOSC to EHOSC*/
void switchFromIHOSCtoEHOSC(void)
{
    /* Use X1/X2(fEHOSC) */
    CG_SetPortM(CG_PORTM_AS_HOSC);
    /* Enable high-speed oscillator */
    CG_SetFosc(CG_FOSC_OSC1, ENABLE);
    /* Select internal(fIHOSC) as warm-up clock */
    CG_SetWarmUpTime(CG_WARM_UP_SRC_OSC2, CG_WUODR_EXT);
    /* Start warm-up */
    CG_StartWarmUp();
    /* Wait until EHOSC become stable */
    while (CG_GetWarmUpState()!=DONE) {
    }
    /* Select external oscillator as High-speed oscillator */
    CG_SetFoscSrc(CG_FOSC_OSC1);
    /* Wait for <OSCSEL> to become "1" */
    while (CG_GetFoscSrc()!=CG_FOSC_OSC1) {
    }
    /* Stop IHOSC */
    CG_SetFosc(CG_FOSC_OSC2, DISABLE);
}

/* Switch over from EHOSC to IHOSC*/
void switchFromEHOSCtoIHOSC(void)
{
    /* Enable high-speed oscillator */
    CG_SetFosc(CG_FOSC_OSC2, ENABLE);
    /* Select enternal(fEHOSC) as warm-up clock */
    CG_SetWarmUpTime(CG_WARM_UP_SRC_OSC1, CG_WUODR_EXT);
    /* Start warm-up */
    CG_StartWarmUp();
    /* Wait until EHOSC become stable */
    while (CG_GetWarmUpState()!=DONE) {
    }
    /* Select ixternal oscillator as High-speed oscillator */
    CG_SetFoscSrc(CG_FOSC_OSC2);
    /* Wait for <OSCSEL> to become 0" */
    while (CG_GetFoscSrc()!=CG_FOSC_OSC2) {
    }
    /* Stop EHOSC */
    CG_SetFosc(CG_FOSC_OSC1, DISABLE);
}
/*********************************** END OF FILE ******************************/
