/** 
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of DSADC_Data_Read demo for the TOSHIBA
 *          'TMPM061' Device Series 
 * @version V0.202
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */


#include "main.h"

#define TMRB_1MS        0x7d0U  /* fsys = fc = 16MHz, fphiT0 = fperiph = fgear = fc */
                                 /* ftmrb = 1/8fphiT0 = 1/8 * 16 = 2MHz, Ttmrb = 0.5us, 1000us/0.5us = 2000 = 0x7D0 */


/**
  * @brief  The main routine of DSADC demo
  * @param  None
  * @retval None
  */
int main(void)
{
    TMRB_InitTypeDef m_tmrb;
    TSB_DSAD_TypeDef *DSADCx = TSB_DSAD0;
    DSADC_InitTypeDef InitStruct;
    uint32_t result = 0U;
    uint8_t i = 0U;
    TSB_DSAD_TypeDef *t_DSADx[] = { TSB_DSAD0, TSB_DSAD1, TSB_DSAD2 };
    f1ms = 0U;
    f1s = 0U;
    Retarget_Init();

    /*TMRB initial */
    m_tmrb.Mode = TMRB_INTERVAL_TIMER;
    m_tmrb.ClkDiv = TMRB_CLK_DIV_8;
    m_tmrb.TrailingTiming = TMRB_1MS;   /* periodic time is 1ms */
    m_tmrb.UpCntCtrl = TMRB_AUTO_CLEAR;
    m_tmrb.LeadingTiming = TMRB_1MS;    /* periodic time is 1ms */

    TMRB_Enable(TSB_TB0);
    TMRB_Init(TSB_TB0, &m_tmrb);
    NVIC_EnableIRQ(INTTB0_IRQn);
    TMRB_SetRunState(TSB_TB0, TMRB_RUN);

    TSB_TEMP->EN = 0x03U;       /*Enable AMP and BGR at same time */

    while (f1ms != 1U) {
    };
    f1ms = 0U;
    for (i = 0U; i < (sizeof(t_DSADx) >> 2U); ++i) {    /*initilize all DSADC channel */
        DSADCx = t_DSADx[i];
        DSADC_SWReset(DSADCx);
        DSADC_SetClkSupply(DSADCx, ENABLE);     /*Clock feed to DSADC */

        InitStruct.Clk = DSADC_FC_DIVIDE_LEVEL_1;
        InitStruct.BiasEn = ENABLE;
        InitStruct.ModulatorEn = ENABLE;
        InitStruct.SyncMode = DSADC_SYNC_MODE;  /*DSADC_SYNC_MODE; */

        InitStruct.Repeatmode = DSADC_REPEAT_MODE;      /*Repeat mode */
        InitStruct.Amplifier = DSADC_GAIN_8x;
        InitStruct.Offset = 0x03U;
        InitStruct.CorrectEn = ENABLE;
        DSADC_Init(DSADCx, &InitStruct);
    }
    DSADC_Start(TSB_DSAD0);
    DSADC_Start(TSB_DSAD1);
    DSADC_Start(TSB_DSAD2);
    while (1U) {
        if (f1s == 1U) {
            f1s = 0u;
            result = DSADC_GetConvertResult(TSB_DSAD0);
            printf("DSAD0: %d\r\n", result);

            result = DSADC_GetConvertResult(TSB_DSAD1);
            printf("DSAD1: %d\r\n", result);

            result = DSADC_GetConvertResult(TSB_DSAD2);
            printf("DSAD2: %d\r\n", result);
        }
    }
}

#ifdef DEBUG
/**
  * @brief  Deal with the error parameter
  * @param  file: Pointer to the file where the error parameter locates
  * @param  line: Number of the line in which the error parameter locates
  * @retval None
  */
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
