/**
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of LCD demo for the
 *          TOSHIBA 'TMPM061' Demo
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* --------------------------------------------------------------------------------
   This demo use an customers LCD,(type: JY09484G from Hebei JIYA Electronic Co.,Ltd. ) 
   the digit in the string 12345678 will be displayed on the digit 8 area one by one 
   in adjustable interval time.
   
   The display order are: 
    1.	display 1 in the right one
    2.	display 12 in right two digit
    3.	display 123 in right three digit
    4.	display 1234 in right four digit
    5.	display 12345 in right five digit
    6.	display 123456 in right six digit
    7.	display 1234567 in right sever digit
    8.	display 12345678 in all the eight digit
    9.	goto step 1 above

 --------------------------------------------------------------------------------*/

/* include file */
#include "tmpm061_lcd.h"
#include "lcd061.h"


struct mySoftTimer {
    uint8_t flag_TimeUp;
    uint8_t flag_Blink;
    uint32_t timer_Cnt;
}softT;

#define DELAYCONST  200000U
static void softTimer_init(void);
static void softTimer_Run(void); 

uint32_t cnt = 0U;
const uint32_t digit_array[] = { 1, 12, 123, 1234, 12345, 123456,1234567,12345678 };

int main()
{
    /* initialize LCD pins */
    lcd_port_init();
    
    /* initialize LCD relative settings */
    lcd_configure_init();
    
    /* initialize soft timer */
    softTimer_init();

    while (1U) {

        softTimer_Run();  
        if(softT.flag_TimeUp) {
            softT.flag_TimeUp = 0;
            
            display_num( digit_array[cnt], 0U);
            if( ++cnt > 7 ) {
                cnt = 0;
            } else {
                /* Do nothing */
            }
        } else {
            /* Do nothing */
        }
    }
}



/**
  * @brief  Initialize a several mS timer. 
  * @param  None
  * @retval None
  */
static void softTimer_init(void)
{
    softT.flag_TimeUp = 0U;
    softT.flag_Blink = 0U;
    softT.timer_Cnt = DELAYCONST;
}  
  

/**
  * @brief  Simulate a several mS timer. 
  * @param  None
  * @retval None
  */
static void softTimer_Run(void)
{
    if( softT.timer_Cnt == 0U ){
        softT.flag_TimeUp = 1U;
        softT.flag_Blink = (uint8_t)~softT.flag_Blink; 
        softT.timer_Cnt = DELAYCONST;
    } else {
        softT.timer_Cnt --;
    }
}


#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
