/**
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of LCD demo for the TMPM061-SK board
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* --------------------------------------------------------------------------------
   This demo use the static LCD on TMPM061-SK board,(type: S5026D with 4 digit '8' ) 
   the digit in the string    0123456789 will be displayed on the LCD one by one 
   in adjustable interval time.
   
   The display order are: 
    1.	display     0 
    2.	display    01 
    3.	display   012 
    4.	display  0123 
    5.	display  123.4 
    6.	display  23.45 
    7.	display  3.456 
    8.	display  4567
    9.	display  5678
    10.	display  6789  
    11.	goto step 1 above

 --------------------------------------------------------------------------------*/

/* include file */
#include "lcd061.h"
   

struct mySoftTimer {
    uint8_t flag_TimeUp;
    uint8_t flag_Blink;
    uint32_t timer_Cnt;
}softT;

#define DELAYCONST  200000U
static void softTimer_init(void);
static void softTimer_Run(void); 

uint8_t cnt = 0U, idx = 0U, dot = 0U;
unsigned char str[] = "   0123456789";

int main()
{
    /* initialize LCD pins */
    lcd_port_init();
    
    /* initialize LCD relative settings */
    LCD_Init();
    
    /* initialize soft timer */
    softTimer_init();

    while (1U) {

        softTimer_Run();  
        if(softT.flag_TimeUp) {
            softT.flag_TimeUp = 0;
            
            if(idx > 3U && idx < 7U){
                dot = 7U - idx;
            } else {
                dot = 0U;
            }
            display_4digit( &str[idx], dot );
            
            idx++;
            if( idx > 9U ) {
                idx = 0U;
            } else {
                /* Do nothing */
            }
        } else {
            /* Do nothing */
        }
    }
}



/**
  * @brief  Initialize a several mS timer. 
  * @param  None
  * @retval None
  */
static void softTimer_init(void)
{
    softT.flag_TimeUp = 0U;
    softT.flag_Blink = 0U;
    softT.timer_Cnt = DELAYCONST;
}  
  

/**
  * @brief  Simulate a several mS timer. 
  * @param  None
  * @retval None
  */
static void softTimer_Run(void)
{
    if( softT.timer_Cnt == 0U ){
        softT.flag_TimeUp = 1U;
        softT.flag_Blink = (uint8_t)~softT.flag_Blink; 
        softT.timer_Cnt = DELAYCONST;
    } else {
        softT.timer_Cnt --;
    }
}


#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
