/**
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of RTC demo for the
 *          TOSHIBA 'TMPM061' Demo
 * @version V0.111
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* include file */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/**
  * @brief  main function.
  * @param  None
  * @retval None
  */

int main(void)
{
#ifdef  M061_SK
    LCD_Init();
#else
    LCD_Configuration();
#endif
    /* enable Fs for RTC */
    if (CG_GetFsState() == DISABLE) {
        CG_SetFs(ENABLE);
    }
    __disable_irq();
    /* enable RTC interrupt */
    NVIC_ClearPendingIRQ(INTRTC_IRQn);

    /* enable RTC interrupt */
    CG_SetSTBYReleaseINTSrc(CG_INT_SRC_RTC, CG_INT_ACTIVE_STATE_FALLING, ENABLE);
    RTC_Init();

    NVIC_EnableIRQ(INTRTC_IRQn);
    __enable_irq();

    /* waiting for RTC register set finish */
    while (fRTC_1HZ_INT != 1U) {
        /* Do nothing */
    }
    fRTC_1HZ_INT = 0U;

    /* Enable RTC Clock function */
    RTC_EnableClock();

    while (1) {
        if (fRTC_1HZ_INT == 1U) {
            fRTC_1HZ_INT = 0U;

            RTC_Display();
        } else {
            /* Do nothing */
        }
    }
}

/**
  * @brief  Deal with the error parameter
  * @param  file: Pointer to the file where the error parameter locates
  * @param  line: Number of the line in which the error parameter locates
  * @retval None
  */

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
