/** 
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of TMRB_PPG demo for the TOSHIBA
 *          'TMPM061' Device Series 
 * @version V0.201
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/**
  * @brief  Delay some time
  * @param  None
  * @retval None
  */
void delay(void)
{
    uint32_t i = 0U;
    for(i = 0U; i<0xFFFFU; i++){
      /* Do nothing */
    }
}

/**
  * @brief  The main function of TMRB_PPG demo
  * @param  None
  * @retval None
  */
int main(void)
{
    uint8_t SW_info;
    uint8_t wait_SW0 = 1U;
    uint8_t Rate = 0U;
    uint32_t LeadingTiming[5] = { 0x384U, 0x2EEU, 0x1F4U, 0xFAU, 0x64U  };       /* leadingtiming: 10%, 25%, 50%, 75%, 90% */

    TMRB_InitTypeDef m_tmrb;
    TMRB_FFOutputTypeDef PPGFFInital;

    SW_Init();
    /* Set PG0 as TB0OUT for PPG output */
    TSB_PG->CR |= 0x01;
    TSB_PG->FR1 |= 0x01;

    m_tmrb.Mode = TMRB_INTERVAL_TIMER;
    m_tmrb.ClkDiv = TMRB_CLK_DIV_8;
    m_tmrb.UpCntCtrl = TMRB_AUTO_CLEAR;
    m_tmrb.TrailingTiming = TMRB0TIME;   /* T = 1ms */
    m_tmrb.LeadingTiming = LeadingTiming[Rate];
    PPGFFInital.FlipflopCtrl = TMRB_FLIPFLOP_CLEAR;
    PPGFFInital.FlipflopReverseTrg = TMRB_FLIPFLOP_MATCH_TRAILINGTIMING | TMRB_FLIPFLOP_MATCH_LEADINGTIMING;

    TMRB_Enable(TSB_TB0);
    TMRB_Init(TSB_TB0, &m_tmrb);
    TMRB_SetFlipFlop(TSB_TB0, &PPGFFInital);
    TMRB_SetDoubleBuf(TSB_TB0, ENABLE); /* enable double buffer */
    TMRB_SetRunState(TSB_TB0, TMRB_RUN);

    while (1) {      
        while (wait_SW0) {
            /* read switch SW0 input */
            SW_info = SW_Get(SW0);
            delay();
            if (SW_info == 1) {
                /* wait for switch SW0 released */
                do {
                    wait_SW0 = 0U;
                    SW_info = SW_Get(SW0);
                    delay();
                } while (SW_info != SWRELEASE);
            }
        }
        wait_SW0 = 1U;     
        Rate++;
        if (Rate >= LEADINGTIMINGMAX) {  /* change leadingtiming rate */
            Rate = LEADINGTIMINGINIT;
        } else {
            /* Do nothing */
        }

        TMRB_ChangeLeadingTiming(TSB_TB0, LeadingTiming[Rate]);   /* switch is High again */
    }
}

/**
  * @brief  Deal with the error parameter
  * @param  file: Pointer to the file where the error parameter locates
  * @param  line: Number of the line in which the error parameter locates
  * @retval None
  */
#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
