/** 
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of SIO_Demo
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

uint8_t SIO0_TxBuffer[BufSize] = "MasterToSlave";
uint8_t SIO3_RxBuffer[BufSize] = { 0U };

uint8_t SIO3_TxBuffer[BufSize] = "SlaveToMaster";
uint8_t SIO0_RxBuffer[BufSize] = { 0U };

volatile uint8_t fSIO0TxOK = 1U;
volatile uint8_t fSIO3TxOK = 1U;
volatile uint8_t gSIO0WrIndex = 0U;
volatile uint8_t gSIO3RdIndex = 0U;
volatile uint8_t gSIO3WrIndex = 0U;
volatile uint8_t gSIO0RdIndex = 0U;

void SIO_Configure(void);

/**
  * @brief  main  function
  * @param  None
  * @retval int
  */
int main(void)
{
    SIO_InitTypeDef SIO0_Init, SIO3_Init;

    /*configure the IO port of SIO */
    SIO_Configure();
    /*configure the SIO0 channel */
    SIO_Enable(SIO0);

    /*initialize the SIO0 struct */
    SIO0_Init.InputClkEdge = SIO_SCLKS_TXDF_RXDR;
    SIO0_Init.IntervalTime = SIO_SINT_TIME_SCLK_1;
    SIO0_Init.TransferMode = SIO_TRANSFER_FULLDPX;
    SIO0_Init.TransferDir = SIO_LSB_FRIST;
    SIO0_Init.Mode = SIO_ENABLE_TX | SIO_ENABLE_RX;
    SIO0_Init.DoubleBuffer = SIO_WBUF_ENABLE;
    SIO0_Init.BaudRateClock = SIO_BR_CLOCK_T4;
    SIO0_Init.Divider = SIO_BR_DIVIDER_2;

    SIO_Init(SIO0, SIO_CLK_BAUDRATE, &SIO0_Init);

    /*configure the SIO3 channel */
    SIO_Enable(SIO3);

    /*initialize the SIO3 struct */
    SIO3_Init.InputClkEdge = SIO_SCLKS_TXDF_RXDR;
    SIO3_Init.TransferMode = SIO_TRANSFER_FULLDPX;
    SIO3_Init.TransferDir = SIO_LSB_FRIST;
    SIO3_Init.Mode = SIO_ENABLE_TX | SIO_ENABLE_RX;
    SIO3_Init.DoubleBuffer = SIO_WBUF_ENABLE;

    SIO_Init(SIO3, SIO_CLK_SCLKINPUT, &SIO3_Init);


    /* enable SIO0 Channel TX interrupt */
    NVIC_EnableIRQ(INTTX0_IRQn);
    /* enable SIO3 Channel RX interrupt */
    NVIC_EnableIRQ(INTRX3_IRQn);

    /* enable SIO3 Channel TX interrupt */
    NVIC_EnableIRQ(INTTX3_IRQn);
    /* enable SIO0 Channel RX interrupt */
    NVIC_EnableIRQ(INTRX0_IRQn);

    while (1) {
        /*SIO3 send data from TXD1 */
        if (fSIO3TxOK == 1U) {
            fSIO3TxOK = 0U;
            SIO_SetTxData(SIO3, SIO3_TxBuffer[gSIO3WrIndex++]);
        } else {
            /*Do Nothing */
        }

        /*SIO0 send data from TXD0 */
        if (fSIO0TxOK == 1U) {
            fSIO0TxOK = 0U;
            SIO_SetTxData(SIO0, SIO0_TxBuffer[gSIO0WrIndex++]);
        } else {
            /*Do Nothing */
        }
       
       /*SIO0 receive data end */
        if (gSIO0RdIndex >= BufSize) {
            fSIO3TxOK = 0U;
            SIO_Disable(SIO3);
        } else {
            /*Do Nothing */
        }
        /*SIO3 receive data end */
        if (gSIO3RdIndex >= BufSize) {
            fSIO0TxOK = 0U;
            SIO_Disable(SIO0);
        } else {
            /*Do Nothing */
        }

    }
}

/**
  * @brief  configure the IO port of SIO
  * @param  None
  * @retval None
  */
void SIO_Configure(void)
{
    /*SET PH0 AS TXD0 */
    TSB_PH->FR1 |= 1U ;
    TSB_PH->CR |=  1U ;
    
    /*SET PH1 AS RXD0 */
    TSB_PH->FR1 |= (1U << 1U);
    TSB_PH->IE |= (1U << 1U);

    /*SET PH2 AS SCLK0 output */
    TSB_PH->FR1 |= (1U << 2U);
    TSB_PH->CR |= (1U << 2U);

    /*SET PI4 AS TXD3 */
    TSB_PI->FR1 |= (1U << 4U);
    TSB_PI->CR |= (1U << 4U);
  
    /*SET PI5 AS RXD3 */
    TSB_PI->FR1 |= (1U << 5U);
    TSB_PI->IE |= (1U << 5U);

    /*SET PI6 AS SCLK3 input */
    TSB_PI->FR1 |= (1U << 6U);
    TSB_PI->IE |= (1U << 6U);

}

/**
  * @brief  Deal with the error parameter
  * @param  file: Pointer to the file where the error parameter locates
  * @param  line: Number of the line in which the error parameter locates
  * @retval None
  */
#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
