/**
 ******************************************************************************
 * @file     sw.c
 * @brief    SW driver for the TOSHIBA 'TMPM061' Device Series
 * @version  V0.101
 * @date     2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 ******************************************************************************
 */


/* Includes ------------------------------------------------------------------*/
#include "sw.h"

/*******************************************************************************
* Function Name  : SW_Init
* Description    : Initialize the GPIO to SW 
* Input          : None 
* Return         : None.
*******************************************************************************/
void SW_Init(void)
{
    uint8_t reg = TSB_SW->IE;
    reg |= SW04_BIT|SW15_BIT|SW26_BIT|SW37_BIT;
    TSB_SW->IE = reg;
    
    reg = TSB_SWS->CR;
    reg |= SWS_BIT;
    TSB_SWS->CR = reg;
    
}

/*******************************************************************************
* Function Name  : SW_Get
* Description    : Get SW Value
* Input          : SW (choose from SW0 ~ SW7).
* Return         : SW value.
*******************************************************************************/
uint8_t SW_Get(uint8_t sw)
{
  
    uint8_t tmp = 0U;
    if (sw < 4) {
        TSB_SWS->DATA &= SWS_BIT_MASK;  //set it as 0
    } else {
        TSB_SWS->DATA |= SWS_BIT;
    }
    switch (sw) {
    case SW0:
    case SW4:
        tmp =  TSB_SW->DATA&SW04_BIT;
        break;
    case SW1:
    case SW5:
        tmp =  TSB_SW->DATA&SW15_BIT;
        break;
    case SW2:
    case SW6:
        tmp =  TSB_SW->DATA&SW26_BIT;
        break;
    case SW3:
    case SW7:
        tmp =  TSB_SW->DATA&SW37_BIT;
        break;
    default:
        tmp = 0U;
        break;
    }    
    if(tmp >0U) tmp = 1U;
    return tmp;

}
